/*************************************************************************************/
/* an example program to demonstrate a basic use of pythia8                          */
/* for a user defined process setup                                                  */
/* this is equivalent to the example in Moses1.0_qqbar2gmZ2mumubar.cxx               */
/*************************************************************************************/
#include "pythiaPlugin.h"

using namespace std;
using namespace Moses1;


int main() {
cout << "Holy Moses..." << endl;
	
	
	cout << "\n\n" << "-------------------Set a user defined process run with Pythia8-------------------" << endl;
	// Pythia generator.
	Pythia pythia;

	//------------------------------------------------------
	// Chosen new PDF set; LHAPDF file name conventions.
	//string pdfSet = "cteq61.LHgrid";
	//string pdfSet = "cteq65.LHgrid";
	//string pdfSet = "cteq65.LHpdf";
	//string pdfSet = "cteq5l.LHgrid";
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq6.LHpdf";
	//string pdfSet = "MRST2001E.LHgrid";
	int pdfMember_int = 0;
	string pdfMember_str = "0";
	stringstream s;
	if (pdfSet=="cteq6.LHpdf" || pdfSet=="cteq65.LHpdf" || pdfSet=="cteq65.LHgrid") {
		cout << "pdf set is: " << pdfSet << endl;
		cout << "please chose a pdf member (0->40)...";
		cin >> pdfMember_int;
		s << pdfMember_int;
		s >> pdfMember_str;
		if (pdfSet=="cteq6.LHpdf" && pdfMember_int == 0) { pdfSet = "cteq6m.LHpdf"; }
		s.clear();
	}
	else if (pdfSet=="MRST2001E.LHgrid") {
		cout << "pdf set is: " << pdfSet << endl;
		cout << "please chose a pdf member (0=central, 1->30=errors)...";
		cin >> pdfMember_int;
		s << pdfMember_int;
		s >> pdfMember_str;
		s.clear();
	}
	//------------------------------------------------------

	//------------------------------------------------------------------------------------------------------------------------------------
	int outFermionID = 6; // top
	int nKKmodes = 1; // number of KK modes

	// A class to generate the process from external matrix element. 
	SigmaProcess* myProcess = new FGB(outFermionID, nKKmodes, pdfSet, pdfMember_int);	

	
	// A class to generate the process from external matrix element. 
//	new2Process* myProcess = new new2Process(outFermionID);
	
//	myProcess->registerEvent(&pythia.event);

	// Hand pointer to Pythia.
	pythia.setSigmaPtr(myProcess);
	cout << "Hand pointer to Pythia" << endl;


	// Chosen new PDF set; LHAPDF file name conventions.
	//string pdfSet = "cteq61.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	//string pdfSet = "MRST2006nnlo.LHgrid";
	pythia.readString("PDF:useLHAPDF = on");
	pythia.readString("PDF:LHAPDFset = " + pdfSet);
	// Allow extrapolation of PDF's beyond x and Q2 boundaries, at own risk.
	// Default behaviour is to freeze PDF's at boundaries.
	pythia.readString("PDF:extrapolateLHAPDF = on");
	// Need to change pT0Ref depending on choice of PDF.
	// One possibility: retune to same <n_charged>.
	//pythia.readString("MultipleInteractions:pT0Ref = 5");

	// Create the dummy Z0 / gamma* particles so they can be inserted in the event record
	// particle: id="23" name="Z0" spinType="3" chargeType="0" colType="0" m0="91.18760" mWidth="2.49520" mMin="10.00000" mMax="0.00000"
	// id:all = name antiName spinType chargeType colType m0 mWidth mMin mMax tau0
	
	pythia.readString("500021:all = gluonAvi void 3 0 0 3000. 120. 10.00000 0.00000");
	pythia.readString("500021:isResonance = false");
	


	// Optionally only compare cross sections.
	string partonLevelSwitch = "n";
	string ISRswitch = "n";
	cout << "switch off PartonLevel:all ? (y/n)......";
	cin >> partonLevelSwitch;
	if (partonLevelSwitch == "y") {
		pythia.readString("PartonLevel:all = off");
		//pythia.readString("PartonLevel:ISR = off");
	}
	else if (partonLevelSwitch == "n") {
		cout << "switch off PartonLevel:ISR ? (y/n)......";
		cin >> ISRswitch;
		if (ISRswitch == "y") {
			pythia.readString("PartonLevel:ISR = off");
		}
		else if (ISRswitch == "n") {
			pythia.readString("PartonLevel:ISR = on");
		}
	}


	
	// phase space cuts
	pythia.readString("PhaseSpace:mHatMin = 1000");
	pythia.readString("PhaseSpace:mHatMax = 5000");

	// Initialization for LHC.
	pythia.init(2212, 2212, 14000.);
	cout << "Initialization for LHC" << endl;

	// List changes.
	pythia.settings.listChanged();
	pythia.particleData.listChanged();

	
	commonOperations commonOps;
	string dirName = "/";	

	//string filePath = commonOps.checkANDsetFilepath("MOSESDATA", "pythia_external_muons.dat");
	//ofstream muFile;
	//muFile.open(filePath.c_str());
	
	string filePath2 = commonOps.checkANDsetFilepath("MOSESDATA", "/eventRecord_Avi_qqbar2G2ttbar.dat");
        ofstream eventRecord;
        eventRecord.open(filePath2.c_str());


	string filePath3 = commonOps.checkANDsetFilepath("MOSESDATA", dirName+"pythia_external_Avi_top.root");
	string treeName = "testTree";
	TFile *treeFile = new TFile(filePath3.c_str(),"recreate");
	TTree *tree = new TTree(treeName.c_str(),treeName.c_str());
	int eventNumber;
	double mHat, thetaHat, cosThetaHat;
	tree->Branch("eventNumber",&eventNumber,"eventNumber/I");
	tree->Branch("mHat",&mHat,"mHat/D");
	tree->Branch("thetaHat",&thetaHat,"thetaHat/D");


	int nEvents = 200;

	cout << "\n\n" << "---------------------------Start the run--------------------------" << endl;
	cout << "\n\nenter the number of MUON-PAIRS to loop over (integer>0, default is 200): ";
	cin >> nEvents;
	cout << "start running and booking events to files..." << endl;
	cout << "\n\n";

	Vec4 p1,p2,pSum,p, pq;
	int idL1=0, idL2=0;

	// run
	int mod = (int)nEvents/10;
	int evt=1;
	bool passCuts = true;
	while (evt <= nEvents) {
		pythia.next();

		eventNumber = evt;
		mHat = pythia.info.mHat();
		//---- in a 2->1->2 process thetaHat is not given in by pythia.info.thetaHat() unlike mHat.------------------
		// therefore we have to calculate it explicitly from the process record (not the event record)
		for (int i=0 ; i<((int)pythia.process.size()-1) ; i++) {
			if (abs(pythia.process[i].id()) == outFermionID && abs(pythia.process[i+1].id()) == outFermionID) {
				idL1 = pythia.process[i].id();
				p1.e(pythia.process[i].e());
				p1.px(pythia.process[i].px());
				p1.py(pythia.process[i].py());
				p1.pz(pythia.process[i].pz());
				idL2 = pythia.process[i+1].id();
				p2.e(pythia.process[i+1].e());
				p2.px(pythia.process[i+1].px());
				p2.py(pythia.process[i+1].py());
				p2.pz(pythia.process[i+1].pz());
			}
		}
		
		pSum = p1 + p2; // proportional to the boost
		if (idL1 > 0) {
			p1.bstback(pSum); // boost back with beta=-veq{pSum}/pSum^0  (beta = -p/E)
			p = p1;
		}
		else if (idL2 > 0) {
			p2.bstback(pSum); // boost back with beta=-veq{pSum}/pSum^0  (beta = -p/E)
			p = p2;
		}
		// calculate cos(theta*) relative to the boost
		cosThetaHat =	( p.px()*pSum.px() + p.py()*pSum.py() + p.pz()*pSum.pz() ) /
				(
					sqrt(p.px()*p.px() + p.py()*p.py() + p.pz()*p.pz()) *
					sqrt(pSum.px()*pSum.px() + pSum.py()*pSum.py() + pSum.pz()*pSum.pz())
				); // cosTheta between the vectors wrt the boost direction.
		//cosThetaHat = p.pz()*pq.pz()/(sqrt(p.px()*p.px() + p.py()*p.py() + p.pz()*p.pz())*abs(pq.pz()));
		//-----------------------------------------------------------------------------------------------------------
                thetaHat = acos(cosThetaHat);
		//thetaHat = pythia.info.thetaHat();
		tree->Fill();
		//muFile << evt << "\t" << pythia.info.mHat() << "\t" << pythia.info.thetaHat() << "\n";
		if (evt%mod==0) {cout << "event: " << evt << "\t\t size=" << pythia.event.size() << endl;}
		if (evt==nEvents) {
			pythia.process.list();
			pythia.event.list(eventRecord);
		}
		if (passCuts) { evt++; }
	}
	//muFile.close();
	eventRecord.close();
	tree->Print();
	treeFile->Write();
	pythia.statistics();
	
	// Done.
	return 0;
}
