/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;

using namespace Moses1;

int main() {
	cout << "Holy Moses..." << endl;
	
	TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
	plotstyle->cd();

	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)


	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with symmetric path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	commonOperations commonOps; // set the class for commonOperations call
	//--------------------------------------------------------------------//


	// for the runs
	dcomplex gplus (0,0);
	dcomplex gminus (0,0);
	dcomplex helicityAmp (0,0);
	double helicityAmpSquared = 0;
	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 100;
	double differentialSquareAmp_asymmetric = 0;
	double differentialSquareAmp_symmetric = 0;
	double totalwidth_ZKK = 0;
	double totalwidth_Z0 = 0;
	double totalwidth_gammaKK = 0;
	double rapidity = 0; // this is the rapidity integral factor (single integral)
	customParticle outFermion = CUSTOMfermions[7]; // propagator will decay into a muon muon_bar pair
	

	double cosTheta_min = -1.;
	double cosTheta_max = +1.;
	double dcosTheta = 1e-3;
	double cosTheta = cosTheta_min;
	double dSigmadcosTheta = 0;
	double mHat = 4000;
	double s_hat = mHat*mHat;
	Int_t nentries = (Int_t)((cosTheta_max-cosTheta_min)/dcosTheta) /*+ (Int_t)dmHat*/;
	cout << "nentries=" << nentries << endl;

	// initialize lha (lhapdf instance)
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with symmetric path and name
	
	// alocate arrays for the graph:
	Double_t *cosTheta_array = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *dSigmadcosTheta_array = (Double_t *)malloc(nentries*sizeof(Double_t));
	

	string treeFilePath = commonOps.checkANDsetFilepath("MOSESDATA","/angular.root");
	TFile *treeFile = new TFile(treeFilePath.c_str(),"RECREATE");
	string treeName = "angularTree";
	TTree *angularTree = new TTree(treeName.c_str(),treeName.c_str());
	
	angularTree->Branch("cosTheta",&cosTheta,"cosTheta/D");
	angularTree->Branch("dSigmadcosTheta",&dSigmadcosTheta,"dSigmadcosTheta/D");


	
	cout << "\n\nRun over cosTheta [" << cosTheta_min << "->" << cosTheta_max << "]:" << endl;
	for (Int_t n=0 ; n<nentries ; n++) { // loop over s_hat
	
		// before each new s_hat it needs to be initialized to '0'
		dSigmadcosTheta = 0;

		// before each new s_hat it needs to be initialized to '0'
		totalwidth_Z0 = 0;
		totalwidth_ZKK = 0;
		totalwidth_gammaKK = 0;

		// calculate the total width of Z/gamma* for each s_hat:
		// calculate the partial width to this pair of ffbar and summ it to get the total width
		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
			switch (CUSTOMfermions[i].pdt_prt.id) {
				case PDTNUE: break;
				case PDTNUMU: break;
				case PDTNUTAU: break;
				default: // gamma can not decay into neutrino pair (while Z can)
					totalwidth_gammaKK += wdt.getpartialWidth(1, s_hat, PDTbosons[1], CUSTOMfermions[i], physPrm); // gammaKK tot width 
					break;
			}
			totalwidth_ZKK += wdt.getpartialWidth(1, s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm); // Z* total width
			totalwidth_Z0  += wdt.getpartialWidth(0, s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm); // Z0 total width
		}

		for (int q=0 ; q<5 ; q++) { // run over all incoming quarks pairs (q+qbar) except for ttbar
			
			differentialSquareAmp_asymmetric = 0; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_symmetric  = 0;
			rapidity	                 = 0;


			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
				for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)

					helicityAmp = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'
					helicityAmpSquared = 0;

					// ---------- SM terms ----------------------------------------------------------------------
					// photon propagator
					helicityAmp += hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[1], physPrm);
					// Z0 propagator
					helicityAmp += hAmp.gethelicityAmplitude(s_hat, totalwidth_Z0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[2], physPrm);
					// ------------------------------------------------------------------------------------------

					// ---------- KK terms ----------------------------------------------------------------------
					for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes
						
						// photon* propagator
						helicityAmp += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_gammaKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[1], physPrm);

						// Z* propagator
						helicityAmp += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_ZKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[2], physPrm);
					}
					// ------------------------------------------------------------------------------------------
					helicityAmpSquared = (s_hat*s_hat) * (1./4.) * real(helicityAmp*conj(helicityAmp));
					
					differentialSquareAmp_asymmetric += helicityAmpSquared * (4.*hIn*hOut) * cosTheta;
					differentialSquareAmp_symmetric  += helicityAmpSquared * (1. + cosTheta*cosTheta);
				} // end hOut
			} // end hIn

			rapidity = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha);
			dSigmadcosTheta += (differentialSquareAmp_symmetric + differentialSquareAmp_asymmetric) * rapidity * 2. / CUSTOMfermions[q].Nc;

		} // end q
		dSigmadcosTheta *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * 2.*physPrm.pi * 2*sqrt(s_hat) * physPrm.GeV2nbarn;
	
		// fill arrays of graphs:
		dSigmadcosTheta_array[n] = dSigmadcosTheta;
		cosTheta_array[n]        = cosTheta;
	
		angularTree->Fill();

		cout << "cosTheta=" << cosTheta << "   dSigmadcosTheta=" << dSigmadcosTheta << "\r" << flush;
		cosTheta += dcosTheta;
	} // end sMat
	angularTree->Print();
	treeFile->Write();

	cout << "\n\n" << "--------------creating graph--------------" << endl;


	string graph = commonOps.checkANDsetFilepath("MOSESDATA","/KK_angular.root");
	TFile *graphFile = new TFile(graph.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph

	TCanvas *c1 = new TCanvas("c1","pp->KK(Z+gamma)->mu-mu+  dsigma/dcosTheta",0,0,1400,800);
//	c1->SetGrid();

/*
	c1->Divide(1,2);
	
	TVirtualPad *pad1=c1->cd(1);
	pad1->SetLogy();

	TVirtualPad *pad2=c1->cd(2);
	pad2->SetGrid();


	c1->cd(1);
	TGraph *crossSection_forward = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_forward);
	TGraph *crossSection_backward = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_backward);
	TGraph *crossSection_FoverB = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_FoverB);
	TGraph *crossSection_symmetric = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_symmetric);
	crossSection_forward->SetTitle("pp->KK(Z+gamma)->mu-mu+: forward, backward, sum & symmetric terms");
	crossSection_forward->SetLineStyle(2);
	crossSection_forward->SetLineColor(kRed);
	crossSection_forward->GetXaxis()->SetTitle("Sqrt(s_hat) [GeV]");
	crossSection_forward->GetYaxis()->SetTitle("forward, backward, sum & symmetric terms");
	crossSection_forward->Draw("AC");
	
	crossSection_backward->SetLineStyle(2);
	crossSection_backward->SetLineColor(kBlue);
	crossSection_backward->Draw("C");
	
        crossSection_FoverB->SetLineStyle(2);
        crossSection_FoverB->SetLineColor(kOrange);
        crossSection_FoverB->Draw("C");

	crossSection_symmetric->SetLineColor(kGreen);
        crossSection_symmetric->Draw("C");

	TLegend *leg = new TLegend(0.7, 0.65, 0.90, 0.90);
	leg->SetFillColor(5);
	leg->AddEntry(crossSection_forward,"forward term","L");
	leg->AddEntry(crossSection_backward,"-1* backward term","L");
	leg->AddEntry(crossSection_FoverB,"F/|B| term","L");
	leg->AddEntry(crossSection_symmetric,"symmetric term","L");
	leg->Draw();
	//----------------------


	c1->cd(2);
*/
	c1->cd();
	TGraph *angular_graph = new TGraph (nentries, cosTheta_array, dSigmadcosTheta_array);
	angular_graph->SetTitle("qqbar->KK(Z+gamma)->mu-mu+: angular distributiom");
	angular_graph->SetLineStyle(2);
	angular_graph->SetLineColor(kRed);
	angular_graph->GetXaxis()->SetTitle("cosTheta");
	angular_graph->GetYaxis()->SetTitle("dSigmadcosTheta");
	angular_graph->Draw("AC");

	
	string epsfile = commonOps.checkANDsetFilepath("MOSESDATA","/KK_angular.eps");
	c1->Print(graph.c_str(),"root");
	c1->Update();
	c1->Print(epsfile.c_str());
	graphFile->Write();

	//--------------------
	cout << "\n" << "done..." << endl;
}

