/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;

using namespace Moses1;

int main() {
	cout << "Holy Moses..." << endl;
	
	TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
	plotstyle->cd();

	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct
	cutsEmulator cuts; // create the cuts emulator instance (input to rapidity)

	

	runPrm.sMax = 10e3*10e3;



	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS}; // glu, gamma, Z0, W+
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	commonOperations commonOps; // set the class for commonOperations call
	//--------------------------------------------------------------------//


	// for the runs
	dcomplex helicityAmp (0,0);
	double helicityAmpSquared = 0;
	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 100;
	double differentialSquareAmp_asymmetric = 0;
	double differentialSquareAmp_symmetric = 0;
	double squareAmp_asymmetric = 0;
	double squareAmp_symmetric = 0;
	double totalwidth_ZKK = 0;
	double totalwidth_Z0 = 0;
	double totalwidth_gammaKK = 0;
	double rapidityAndAngular_asymmetric = 0; // this is the rapidity integral factor (single integral)
	double rapidityAndAngular_symmetric = 0;
	customParticle outFermion = CUSTOMfermions[7]; // propagator will decay into a muon muon_bar pair
	
	double Afb;
	
	double mHat_min = 20.;
	double mHat_max = 150.;
	double dmHat = 1./*1.*/;
	double mHat = mHat_min;
	double s_hat = mHat_min*mHat_min;
	Int_t nentries = (Int_t)((mHat_max-mHat_min)/dmHat) /*+ (Int_t)dmHat*/;
	cout << "nentries=" << nentries << endl;

	// initialize lha (lhapdf instance)
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name
	
	// alocate arrays for the graph:
	Double_t *Sqrt_s_hat = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_asymmetric = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_symmetric = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Afb_array = (Double_t *)malloc(nentries*sizeof(Double_t));


	string treeFilePath = commonOps.checkANDsetFilepath("MOSESDATA","/asymmetry.root");
	TFile *treeFile = new TFile(treeFilePath.c_str(),"RECREATE");
	string treeName = "asymmetryTree";
	TTree *asymmetryTree = new TTree(treeName.c_str(),treeName.c_str());
	
	asymmetryTree->Branch("mHat",&mHat,"mHat/D");
	asymmetryTree->Branch("dsigmadmHat_forward",&squareAmp_asymmetric,"squareAmp_asymmetric/D");
	asymmetryTree->Branch("dsigmadmHat_backward",&squareAmp_symmetric,"squareAmp_symmetric/D");
	asymmetryTree->Branch("Afb",&Afb,"Afb/D");

	double average_Afb = 0;
	int    numberOfpoints = 0;

	bool asymmetric = true;
	bool symmetric = false;

	cout << "\n\nRun over s_hat ([" << mHat_min << "->" << mHat_max << "]GeV)^2:" << endl;
	for (Int_t n=0 ; n<nentries ; n++) { // loop over s_hat
	
		// before each new s_hat it needs to be initialized to '0'
		squareAmp_asymmetric = 0.;
		squareAmp_symmetric  = 0.;

		// before each new s_hat it needs to be initialized to '0'
		totalwidth_Z0 = 0;

		// calculate the total width of Z/gamma* for each s_hat:
		// calculate the partial width to this pair of ffbar and summ it to get the total width
		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
			totalwidth_Z0  += wdt.getpartialWidth(0, s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm); // Z0 total width
		}


		for (int q=0 ; q<5 ; q++) { // run over all incoming quarks pairs (q+qbar) except for ttbar
			
			differentialSquareAmp_asymmetric = 0; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_symmetric  = 0;


			rapidityAndAngular_asymmetric = 0;
			rapidityAndAngular_asymmetric = 0;


			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
				for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)

					helicityAmp = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'
					helicityAmpSquared = 0;

					// ---------- SM terms ----------------------------------------------------------------------
					// photon propagator
					helicityAmp = hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[1], physPrm);
					// Z0 propagator
					helicityAmp += hAmp.gethelicityAmplitude(s_hat, totalwidth_Z0, hIn, hOut,
										CUSTOMfermions[q], outFermion,
										PDTbosons[2], physPrm);
					// ------------------------------------------------------------------------------------------
				
					// ---------- KK terms ----------------------------------------------------------------------
					for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes

						totalwidth_ZKK = 0;
						totalwidth_gammaKK = 0;
						// calculate the total width of Z/gamma* for each s_hat:
						// calculate the partial width to this pair of ffbar and summ it to get the total width
						for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
							switch (CUSTOMfermions[i].pdt_prt.id) {
								case PDTNUE: break;
								case PDTNUMU: break;
								case PDTNUTAU: break;
								default: // gamma can not decay into neutrino pair (while Z can)
									totalwidth_gammaKK += wdt.getpartialWidth(KKmode,s_hat,PDTbosons[1],
												CUSTOMfermions[i],physPrm); // gammaKK tot width
									break;
							}
							totalwidth_ZKK += wdt.getpartialWidth(KKmode,s_hat,PDTbosons[2],CUSTOMfermions[i],physPrm); // Z*
						}
					
						// photon* propagator
						helicityAmp += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_gammaKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[1], physPrm);
						// Z* propagator
						helicityAmp += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_ZKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[2], physPrm);
					}
					// ------------------------------------------------------------------------------------------
				
					helicityAmpSquared = (s_hat*s_hat) * (1./4.) * norm(helicityAmp)/*abs(helicityAmp*conj(helicityAmp))*/;
					
					differentialSquareAmp_asymmetric += 4.*hIn*hOut * helicityAmpSquared;
					differentialSquareAmp_symmetric += helicityAmpSquared;
				} // end hOut
			} // end hIn
			
			rapidityAndAngular_asymmetric = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha,cuts,
							differentialSquareAmp_symmetric,differentialSquareAmp_asymmetric,asymmetric);
			rapidityAndAngular_symmetric = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],
							runPrm,lha,cuts,differentialSquareAmp_symmetric,differentialSquareAmp_asymmetric,symmetric);

			squareAmp_asymmetric += rapidityAndAngular_asymmetric * 2. / CUSTOMfermions[q].Nc;
			squareAmp_symmetric  += rapidityAndAngular_symmetric * 2. / CUSTOMfermions[q].Nc;

		} // end q
		squareAmp_asymmetric  *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * 2.*physPrm.pi * 2*sqrt(s_hat) * physPrm.GeV2nbarn; 
		squareAmp_symmetric   *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * 2.*physPrm.pi * 2*sqrt(s_hat) * physPrm.GeV2nbarn;
		
		
		Afb = squareAmp_asymmetric / squareAmp_symmetric ;

		// fill arrays of graphs:
		Sqare_amp_asymmetric[n]  = squareAmp_asymmetric;
		Sqare_amp_symmetric[n]   = squareAmp_symmetric;
		Afb_array[n]  = Afb;
		Sqrt_s_hat[n] = mHat;
	
		asymmetryTree->Fill();
		if (mHat>=75. && mHat<=115.) {
			average_Afb += Afb;
			numberOfpoints ++;
		}

		cout << "sqrt(s_hat)=" << mHat << "   Afb=" << Afb << "\r" << flush;
		mHat += dmHat;
		s_hat = mHat*mHat;

	} // end sMat
	asymmetryTree->Print();
	treeFile->Write();

	average_Afb /= numberOfpoints;
	cout << "\n\n" << "average Afb in [20GeV,115GeV] is:   Afb_av = " << average_Afb << endl;


	cout << "\n\n" << "--------------creating graph--------------" << endl;


	string graph = commonOps.checkANDsetFilepath("MOSESDATA","/KK_asymetry.root");
	TFile *graphFile = new TFile(graph.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph

	TCanvas *c1 = new TCanvas("c1","pp->KK(Z+gamma)->mu-mu+  dsigma/dsHat cuts influences",0,0,1400,800);
	c1->Divide(1,2);
	
	TVirtualPad *pad1=c1->cd(1);
	pad1->SetLogy();

	TVirtualPad *pad2=c1->cd(2);
	pad2->SetGrid();


	c1->cd(1);
	TGraph *crossSection_asymmetric = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_asymmetric);
	TGraph *crossSection_symmetric = new TGraph (nentries, Sqrt_s_hat, Sqare_amp_symmetric);
	//crossSection_asymmetric->SetMaximum(1.25*crossSection_asymmetric->GetMaximum());
	//crossSection_asymmetric->SetMinimum(0.25*crossSection_asymmetric->GetMinimum());

	crossSection_asymmetric->SetTitle("pp->KK(Z+gamma)->mu-mu+: symmetric & asymmetric terms");
	crossSection_asymmetric->SetLineStyle(2);
	crossSection_asymmetric->SetLineColor(kRed);
	crossSection_asymmetric->GetXaxis()->SetTitle("Sqrt(s_hat) [GeV]");
	crossSection_asymmetric->GetYaxis()->SetTitle("symmetric & asymmetric terms");
	crossSection_asymmetric->Draw("AC");
	
	crossSection_symmetric->SetLineStyle(1);
	crossSection_symmetric->SetLineColor(kGreen);
        crossSection_symmetric->Draw("C");

	TLegend *leg = new TLegend(0.7, 0.65, 0.90, 0.90);
	leg->SetFillColor(5);
	leg->AddEntry(crossSection_asymmetric,"asymmetric term","L");
	leg->AddEntry(crossSection_symmetric,"symmetric term","L");
	leg->Draw();
	//----------------------


	c1->cd(2);
//	c1->cd();
	TGraph *asymmetry_graph = new TGraph (nentries, Sqrt_s_hat, /*Sqare_amp_symmetric*/ Afb_array);
	asymmetry_graph->SetTitle("qqbar->KK(Z+gamma)->mu-mu+: forward-backward asymmetry");
	asymmetry_graph->SetLineStyle(2);
	asymmetry_graph->SetLineColor(kRed);
	asymmetry_graph->GetXaxis()->SetTitle("Sqrt(s_hat) [GeV]");
	asymmetry_graph->GetYaxis()->SetTitle("Afb");
	asymmetry_graph->Draw("AC");
	//----------------------
	
	string epsfile = commonOps.checkANDsetFilepath("MOSESDATA","/KK_asymmetry.eps");
	c1->Print(graph.c_str(),"root");
	c1->Update();
	c1->Print(epsfile.c_str());
	graphFile->Write();

	//--------------------
	cout << "\n" << "done..." << endl;
}

