/********************************************************************************************/
/* an example program to demonstrate (in short description) the building of a cross section */
/********************************************************************************************/
//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;

using namespace Moses1;

int main(int argc, char* argv[]) {
	cout << "Holy Moses..." << endl;
	
	TStyle* plotstyle = (TStyle*)gROOT->GetStyle("Plain");
	plotstyle->cd();

	// declare the application
	TApplication theApp("App", &argc, argv);

	commonOperations commonOps; // set the class for commonOperations call
	// set styles
	commonOps.SetOKStyle();


	cout << "\n\n" << "-------------------Set user defined parameters-------------------" << endl;
	parameters prm; // create an instance of the parameters class
	physicalParameters physPrm; // create an instance of the physical parameters struct
	runParameters runPrm; // create an instance of the run parameters struct
	prm.setParameters(physPrm); // initialize the new instance of the physical parameters struct
	prm.setParameters(runPrm); // initialize the new instance of the run parameters struct


	cout << "\n\n" << "--------------Get ALL particle properties in a table--------------" << endl;

	PDTparticle pdt_prt;
	pdt_prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name

	cout << "\n\n" << "-------Set all involved particle properties into 2 vectors--------" << endl;
	bool print = false;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	// fill all quarks and leptons properties in a single vector
	pdt_prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);

	// do the same for PDTbosons:
	int idBosons[] = {PDTGLU,PDTGAMMA,PDTZ,PDTWPLUS,PDTZPRIME0}; // glu, gamma, Z0, W+, Z'0
	vector<pdtParticle> PDTbosons; // set the pdtParticle properties structs vector
	pdt_prt.getAll(idBosons, sizeof(idBosons)/sizeof(*idBosons), PDTbosons, print); // fill all PDTbosons properties in a single vector


	cout << "\n\n" << "-------Set some more properties manually into the 2 vectors-------" << endl;
	// add to the previously created vector of PDT particle 2 more properties to each properties struct:
	// 1) weakI3, 2) Nc (Number of possible colors)
	// see: mosesParticle.cxx for more details of how this is actually being done
	vector<customParticle> CUSTOMfermions; // define a vector of custom particles
	// put only the selected pdtParticle structs from the PDTfermions vector in the CUSTOMparticle and add more properties:
	CUSTOMparticle custom_prt; // this is only the class instance
	custom_prt.setCUSTOMparticle(PDTfermions, CUSTOMfermions);
	// from now on, have to use the new vector in order to get the custom properties.
	// the PDT vector also exist and can be used.
	// note that the bosons vector is not modified here


	// create relevant classes instances
	couplings cplng; // set the class for couplings call
	partialWidth wdt; // set the class for partialWidth call
	helicityAmplitude hAmp; // set the class for helicityAmplitude call
	rapidity rap;  // set the class for rapidity call
	lhapdf lha; // set the class for lhapdf call
	newZresonance zPrime("chi");
	bool doSMcouplings = true;

	// initialize lha (lhapdf instance)
	string pdfSet = "MRST2001lo.LHgrid";
	//string pdfSet = "cteq5l.LHgrid";
	int subset = 0; // it is usually set to '0'
	lha.setlhapdf("LHAPDFSETS",pdfSet,subset); // can also initialize with full path and name

	customParticle outFermion = CUSTOMfermions[7]; // propagator will decay into a muon muon_bar pair
	// the integral over all decay angles
	// [1] theta: distributing like ~ (1+/-cos(theta))^2
	// [2] phi: distributing uniformly
	double angularIntegral = 2*physPrm.pi * 8/3;
	int nMax_ysegments = 100; // for the integration on the rapidity
	int nKKmodes = 100;
	double mZ0 = PDTbosons[2].mass;
	Double_t dmHat = 1.;


	double CMenergy = 10e3;
        cout << "LHC CM energy[GeV]=";
        cin >> CMenergy;
	stringstream strm;
	string sCME = "";
	strm << CMenergy/1000.;
	strm >> sCME;

	double lum = 300;
	cout << "LHC integrated luminosity[pb^-1]=";
	cin >> lum;
	stringstream strm2;
	string sLum = "";
	strm2 << lum;
	strm2 >> sLum;

	int peakOrDip = 0;
	string sPeakOrDip = "";
	cout << "choose to run over the PEAK[m*/2 -> 3m*/2](0),  the DIP[m*/8 -> 5m*/8](1)  or both(2)...";
	cin >> peakOrDip;
	if (peakOrDip==1) {
		cout << "you chose dip" << endl;
		sPeakOrDip = "onDip";
	}
	else if (peakOrDip==0) {
		cout << "you chose peak" << endl;
		sPeakOrDip = "onPeak";
	}
	else if (peakOrDip==2) {
		cout << "you chose both dip and peak" << endl;
		sPeakOrDip = "onPeakAndDip";
	}
	else {
		cout << "you didn't choose, set to default (peak)" << endl;
		peakOrDip = 0;
		sPeakOrDip = "onPeak";
	}	
	
	
	Double_t invPb2invNb = 1e3;
	Double_t events_SM = 0.;
	Double_t events_E6 = 0.;
	Double_t events_KK = 0.;


	string dirName = "/KKvsZE6vsSMstudy/";
	string rootFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_sigmaTotalVSmass_theory_"+sCME+"TeV_"+sPeakOrDip+".root");
	string epsFileName = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_sigmaTotalVSmass_theory_"+sCME+"TeV_"+sPeakOrDip+".eps");
	TFile *rootFile = new TFile(rootFileName.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph
	TCanvas *c1 = new TCanvas("c1","",0,0,1400,800);
        c1->SetLogy();

	string rootFileName2 = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_significance_theory_"+sCME+"TeV_"+sLum+"invPb_"+sPeakOrDip+".root");
	string epsFileName2 = commonOps.checkANDsetFilepath("MOSESDATA",dirName+"KKvsZE6vsSM_significance_theory_"+sCME+"TeV_"+sLum+"invPb_"+sPeakOrDip+".eps");
	TFile *rootFile2 = new TFile(rootFileName.c_str(),"RECREATE"); // new root file for the cross section + rapidity graph
	TCanvas *c2 = new TCanvas("c2","",0,0,1400,800);
	c2->SetLogy();

	Double_t dmStar = 200.; // GeV step
	Double_t mStar_min = 1e3;
	Double_t mStar_max = 6e3;
	Int_t npoints = (Int_t)((mStar_max-mStar_min)/dmStar) + 1;
	cout << "step size in m^* is " << dmStar << " [GeV], with " << npoints << " steps (from " << mStar_min << " to " << mStar_max << ")." << endl;
	// alocate arrays for the graph:
	Double_t *mStarArray = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *totalSigma_SM = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *totalSigma_KK = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *totalSigma_E6 = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *significance_E6 = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *significance_KK = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *fiveSigmaLine = (Double_t *)malloc(npoints*sizeof(Double_t));
	
	Double_t *significance_E6_peak = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *significance_KK_peak = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *significance_E6_dip  = (Double_t *)malloc(npoints*sizeof(Double_t));
	Double_t *significance_KK_dip  = (Double_t *)malloc(npoints*sizeof(Double_t));

	//--------------------------------------------------------------------//

for (Int_t p=0 ; p<=1 ; p++)
{
Double_t mStar = mStar_min;
for(Int_t k=0 ; k<npoints ; k++)
{
	//---------------------------------------------//
	/*****/ runPrm.sMax = CMenergy*CMenergy;  /*****/
	/*****/ physPrm.mStar = mStar;            /*****/
	/*****/ physPrm.mZprime = mStar;          /*****/
	//---------------------------------------------//
	
	dcomplex fullAmp_SM (0,0);
	dcomplex fullAmp_KK (0,0);
	dcomplex fullAmp_Zprime (0,0);
	double differentialSquareAmp_SM = 0;
	double differentialSquareAmp_KK = 0;
	double differentialSquareAmp_Zprime = 0;
	double squareAmp_SM = 0;
	double squareAmp_KK = 0;
	double squareAmp_Zprime = 0;
	double totalwidth_ZKK = 0;
	double totalwidth_Z0 = 0;
	double totalwidth_Zprime = 0;
	double totalwidth_gammaKK = 0;
	double qqbarRapidity_shat = 0; // this is the rapidity integral factor (single integral)
	

	Double_t mMin = mStar - mStar/2.;
	Double_t mMax = mStar + mStar/2.;
	if (peakOrDip==1)
	{
		mMin = 200.;//mStar/8.
		mMax = mStar/2.; //5.*mStar/8.
	}
	else if (peakOrDip==2)
	{
		if(p==0) { // dip
			mMin = 200.;
			mMax = mStar/2.;
		}
		else if (p==1) { // peak
			mMin = mStar/2.;
			mMax = 3.*mStar/2.;
		}
	}
	else
	{
		mMin = mStar/2.;
		mMax = 3.*mStar/2.;
	}

	Int_t nentries = (Int_t)((mMax-mMin)/dmHat) + 1;
	cout << "\n";
	cout << "mStar=" << mStar << ", width=" << totalwidth_ZKK << ", [mMin=" << mMin << ", mMax=" << mMax << "]" << endl;

	double mHat = (double)mMin;
        double s_hat = (double)(mMin*mMin);
	
	// alocate arrays for the graph:
	Double_t *Sqrt_s_hat = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_SM = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_KK = (Double_t *)malloc(nentries*sizeof(Double_t));
	Double_t *Sqare_amp_Zprime = (Double_t *)malloc(nentries*sizeof(Double_t));
	
	for (Int_t n=0 ; n<nentries ; n++) { // loop over s_hat
	
		squareAmp_SM = 0; // before each new s_hat it needs to be initialized to '0'
		squareAmp_KK = 0;
		squareAmp_Zprime = 0;
		totalwidth_Z0 = 0; // before each new s_hat it needs to be initialized to '0'
		totalwidth_Zprime = 0;

		// calculate the total width of Z/gamma* for each s_hat:
		// calculate the partial width to this pair of ffbar and summ it to get the total width
		for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
			//PDTbosons[2].mass = mZ0;
			totalwidth_Z0     += wdt.getpartialWidth(0, s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm); // Z0 total width
			//PDTbosons[2].mass = mStar;
			totalwidth_Zprime += wdt.getpartialWidth(s_hat, PDTbosons[2], CUSTOMfermions[i], physPrm, (double)mStar); // Z0 total width
			//totalwidth_Zprime += zPrime.getZprimeWidth(doSMcouplings, CUSTOMfermions[i]);
		}
		PDTbosons[2].mass = mZ0;


		for (int q=0 ; q<5 ; q++) { // run over all incoming quarks pairs (q+qbar) except for ttbar
			
			differentialSquareAmp_SM = 0; // before each new q it needs to be initialized to '0'
			differentialSquareAmp_KK = 0;
			differentialSquareAmp_Zprime = 0;

			for (double hIn=-0.5 ; hIn<=0.5 ; hIn++) { // loop over incoming helicity states (all quarks)
				for (double hOut=-0.5 ; hOut<=0.5 ; hOut++) { // loop over outgoing helicity states (muon only)

					fullAmp_SM     = dcomplex(0,0); // before each helicity state this has to be initialized to '(0,0)'
					fullAmp_KK     = dcomplex(0,0);
					fullAmp_Zprime = dcomplex(0,0);

					// ---------- SM terms ----------------------------------------------------------------------
					// photon propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, 0, hIn, hOut,
										CUSTOMfermions[q], outFermion, PDTbosons[1], physPrm);
					// Z0 propagator
					fullAmp_SM += hAmp.gethelicityAmplitude(s_hat, totalwidth_Z0, hIn, hOut,
										CUSTOMfermions[q], outFermion, PDTbosons[2], physPrm);
					// ------------------------------------------------------------------------------------------


					// ---------- KK terms ----------------------------------------------------------------------
					fullAmp_KK = fullAmp_SM;
					for (int KKmode=1 ; KKmode<=nKKmodes ; KKmode++) { // loop over all KK modes
						totalwidth_gammaKK = 0.;
						totalwidth_ZKK     = 0.;
						for (int i=0 ; i<(int)CUSTOMfermions.size() ; i++) {
							switch (CUSTOMfermions[i].pdt_prt.id) {
								case PDTNUE: break;
								case PDTNUMU: break;
								case PDTNUTAU: break;
								default: // gamma can not decay into neutrino pair (while Z can)
								totalwidth_gammaKK += wdt.getpartialWidth(KKmode, s_hat, PDTbosons[1],
										     			  CUSTOMfermions[i], physPrm); // gammaKK tot width
								break;
							}
							totalwidth_ZKK  += wdt.getpartialWidth(KKmode, s_hat, PDTbosons[2],
									   		       CUSTOMfermions[i], physPrm); // Z* total width
						}
						
						// photon* propagator
						fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_gammaKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[1], physPrm);

						// Z* propagator
						fullAmp_KK += hAmp.gethelicityAmplitude(KKmode, s_hat, totalwidth_ZKK, hIn, hOut,
											CUSTOMfermions[q], outFermion,
											PDTbosons[2], physPrm);
					}
					// ------------------------------------------------------------------------------------------


					// ---------- Z' terms ----------------------------------------------------------------------
					fullAmp_Zprime = zPrime.getSquaredAmplitude(hIn,hOut, s_hat, mStar, CUSTOMfermions[q], outFermion,
										    totalwidth_Z0, totalwidth_Zprime, doSMcouplings);
					// ------------------------------------------------------------------------------------------
					

					// get the differential square amplitude: |fullAmp|^2
					differentialSquareAmp_SM     += (s_hat*s_hat) * (1./4.) * real( fullAmp_SM*conj(fullAmp_SM) );
					differentialSquareAmp_KK     += (s_hat*s_hat) * (1./4.) * real( fullAmp_KK*conj(fullAmp_KK) );
					differentialSquareAmp_Zprime += (s_hat*s_hat) * (1./4.) * real( fullAmp_Zprime*conj(fullAmp_Zprime) );
				} // end hOut
			} // end hIn
			
			//cout << "totalwidth_Zprime=" << totalwidth_Zprime << endl;
			//cout << "totalwidth_ZKK=" << totalwidth_ZKK << endl;

			qqbarRapidity_shat = rap.simpsonIntegral(nMax_ysegments,s_hat,CUSTOMfermions[q],runPrm,lha); // the qqbar rapidity integral factor
			// [1] multyply by the rapidity integral factor of the corresponding incoming partons (q+qbar)
			// [2] divide by the number of colors of q since the PDF functions return 9 color combinations
			//     and here only 3 are valid sice the propogator is colorless
			// [3] miltiply by 2 to account the 2 possible origins of the q or the qbar in the beams
			differentialSquareAmp_SM     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
			differentialSquareAmp_KK     *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;
			differentialSquareAmp_Zprime *= 2.*qqbarRapidity_shat/CUSTOMfermions[q].Nc;

			squareAmp_SM     += differentialSquareAmp_SM;  // only then add the differential square amplitude to the full one
			squareAmp_KK     += differentialSquareAmp_KK;
			squareAmp_Zprime += differentialSquareAmp_Zprime;

		} // end q

		// multiply by the coefficients appearing before the squared helicity amplitude
		squareAmp_SM     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
		squareAmp_KK     *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
		squareAmp_Zprime *= physPrm.alpha*physPrm.alpha/(4.*s_hat) * outFermion.Nc * angularIntegral;
			// [squareAmp] = [1/GeV^4]
		squareAmp_SM     *= 2*sqrt(s_hat); // dSigma(s_hat) -> dSigma(sqrt(s_hat)) Jacobian of the transformation
		squareAmp_KK     *= 2*sqrt(s_hat);
		squareAmp_Zprime *= 2*sqrt(s_hat);
			// [squareAmp] = [1/GeV^3]
		squareAmp_SM     *= physPrm.GeV2nbarn; // to convert the units to nb/GeV (originally was nb/GeV^2 but here it is nb/GeV)
		squareAmp_KK     *= physPrm.GeV2nbarn;
		squareAmp_Zprime *= physPrm.GeV2nbarn;
			// [squareAmp] = [nb/GeV]
		// see more details about the conversion using the method "prm.printUnits()" in the end of the loop. 

		// fill arrays of graphs:
		Sqare_amp_SM[n]     = squareAmp_SM;
		Sqare_amp_KK[n]     = squareAmp_KK;
		Sqare_amp_Zprime[n] = squareAmp_Zprime;
		Sqrt_s_hat[n] = mHat;

		//cout << "n:" << n << ", m^*=" << mStar << ", Z'width=" << totalwidth_Zprime << ", Z*width=" << totalwidth_ZKK << ", significance_Z'=" << significance_E6[k] << ", significance_KK=" << significance_KK[k] << ", sqrt(s_hat)=" << mHat << "\r" << flush;
		mHat += dmHat;
		s_hat = mHat*mHat;

	} // end sMat
	cout << endl; 

        // calculate the integral of the theoretical line (simpson integration method)
        Double_t integrand_SM = 0;
        Double_t integrand_KK = 0;
        Double_t integrand_E6 = 0;

        Double_t sigmaTot_SM = 0;
        Double_t sigmaTot_KK = 0;
        Double_t sigmaTot_E6 = 0;

	int nSegments = (int)nentries-1;
	int i=0;
	Double_t h = dmHat/3;
        while (i<=nSegments) {
                // calculate the integrand:
                integrand_SM = Sqare_amp_SM[i];
                integrand_KK = Sqare_amp_KK[i];
                integrand_E6 = Sqare_amp_Zprime[i];
                // simpson:
                if (i==0 || i==nSegments) {
                        sigmaTot_SM += integrand_SM;
                        sigmaTot_KK += integrand_KK;
                        sigmaTot_E6 += integrand_E6;
                }
                else if (i%2==0 && i!=0 && i!=nSegments) {
                        sigmaTot_SM += 2*integrand_SM;
                        sigmaTot_KK += 2*integrand_KK;
                        sigmaTot_E6 += 2*integrand_E6;
                }
		else {
                        sigmaTot_SM += 4*integrand_SM;
                        sigmaTot_KK += 4*integrand_KK;
                        sigmaTot_E6 += 4*integrand_E6;
                }
                // propagate i:
                i++;
        }
        sigmaTot_SM *= h; // [nb]
        sigmaTot_KK *= h; // [nb]
        sigmaTot_E6 *= h; // [nb]

	// fill the array
	mStarArray[k]    = mStar;
	totalSigma_SM[k] = sigmaTot_SM;
	totalSigma_E6[k] = sigmaTot_E6;
	totalSigma_KK[k] = sigmaTot_KK;
	
	events_SM = sigmaTot_SM * (lum * invPb2invNb);
	events_E6 = sigmaTot_E6 * (lum * invPb2invNb);
	events_KK = sigmaTot_KK * (lum * invPb2invNb);


	// in the dip area, the SM is always higher than the KK or Z'
	if (peakOrDip==1) {
		significance_E6[k] = (events_SM-events_E6)/sqrt(events_SM);
		significance_KK[k] = (events_SM-events_KK)/sqrt(events_SM);
		fiveSigmaLine[k]   = 5.;
	}
	else if (peakOrDip==2) {
		if (p==0) { // dip
			significance_E6_dip[k]  = (events_SM-events_E6)/sqrt(events_SM);
			significance_KK_dip[k]  = (events_SM-events_KK)/sqrt(events_SM);
			cout << "m^*=" << mStar << ", significance_E6_dip[" << k << "]=" << significance_E6_dip[k] << endl;
			cout << "m^*=" << mStar << ", significance_KK_dip[" << k << "]=" << significance_KK_dip[k] << "\n" << endl;
		}
		else if (p==1) { // peak
			significance_E6_peak[k] = (events_E6-events_SM)/sqrt(events_SM);
        	        significance_KK_peak[k] = (events_KK-events_SM)/sqrt(events_SM);
			cout << "m^*=" << mStar << ", significance_E6_peak[" << k << "]=" << significance_E6_peak[k] << endl;
			cout << "m^*=" << mStar << ", significance_KK_peak[" << k << "]=" << significance_KK_peak[k] << "\n" << endl;
			fiveSigmaLine[k]   = 5.; // enough one time
		}
	}
	else {
		significance_E6[k] = (events_E6-events_SM)/sqrt(events_SM);
		significance_KK[k] = (events_KK-events_SM)/sqrt(events_SM);
		cout << "m^*=" << mStar << ", significance_E6[" << k << "]=" << significance_E6[k] << endl;
		cout << "m^*=" << mStar << ", significance_KK[" << k << "]=" << significance_KK[k] << "\n" << endl;
		fiveSigmaLine[k]   = 5.;
	}


	mStar += dmStar;
}
}// end loop on p
	
	TGraph *crossSection_SM = new TGraph (npoints, mStarArray, totalSigma_SM);
	TGraph *crossSection_E6 = new TGraph (npoints, mStarArray, totalSigma_E6);
	TGraph *crossSection_KK = new TGraph (npoints, mStarArray, totalSigma_KK);

	TGraph *significanceGraph_E6 = NULL;
	TGraph *significanceGraph_KK = NULL;
	TGraph *significanceGraph_E6_peak = NULL;
        TGraph *significanceGraph_KK_peak = NULL;
	TGraph *significanceGraph_E6_dip = NULL;
        TGraph *significanceGraph_KK_dip = NULL;
	
	if (peakOrDip==2) {
		significanceGraph_E6_peak = new TGraph (npoints, mStarArray, significance_E6_peak);
		significanceGraph_KK_peak = new TGraph (npoints, mStarArray, significance_KK_peak);
		significanceGraph_E6_dip = new TGraph (npoints, mStarArray, significance_E6_dip);
		significanceGraph_KK_dip = new TGraph (npoints, mStarArray, significance_KK_dip);
	}
	else {
		significanceGraph_E6 = new TGraph (npoints, mStarArray, significance_E6);
		significanceGraph_KK = new TGraph (npoints, mStarArray, significance_KK);
	}
	

	TGraph *fiveSigmaLineGraph   = new TGraph (npoints, mStarArray, fiveSigmaLine);


	crossSection_SM->SetTitle("");
	crossSection_SM->SetLineStyle(5);
	crossSection_SM->GetXaxis()->SetTitle("m^{*} [GeV]");
	string sigmaTitle = "";
	sigmaTitle = "#sigma_{peak}#left(#mu^{+}#mu^{-}#right) [nb]";
	crossSection_SM->GetYaxis()->SetTitle(sigmaTitle.c_str());
	//crossSection_SM->SetMaximum(1.05*crossSection_KK->GetMaximum());
	crossSection_SM->SetMaximum(1e-2);
	crossSection_SM->Draw("AC");
	
	crossSection_E6->SetLineStyle(3);
	crossSection_E6->Draw("C");

	crossSection_KK->SetLineStyle(1);
	crossSection_KK->Draw("C");

	TLegend *leg = new TLegend(0.6, 0.7, 0.98, 0.9);
	leg->AddEntry(crossSection_SM,"SM","L");
	leg->AddEntry(crossSection_E6,"Z'_{SM}","L");
	leg->AddEntry(crossSection_KK,"KK","L");
	leg->Draw();

	c1->Print(rootFileName.c_str(),"root");
        c1->Update();
        c1->Print(epsFileName.c_str());
        rootFile->Write();


	TLegend *leg2 = NULL;
	if (peakOrDip==2) {
		significanceGraph_KK_peak->SetMinimum(1.);
		significanceGraph_KK_peak->SetMaximum(1.e3);

		// peak
		significanceGraph_KK_peak->SetTitle("");
		significanceGraph_KK_peak->SetLineStyle(1);
		significanceGraph_KK_peak->GetXaxis()->SetTitle("m^{*} [GeV]");
		sigmaTitle = "Significance";
		significanceGraph_KK_peak->GetYaxis()->SetTitle(sigmaTitle.c_str());
		significanceGraph_KK_peak->Draw("AC");
	
		significanceGraph_E6_peak->SetLineStyle(5);
		significanceGraph_E6_peak->Draw("C");

		// dip
		significanceGraph_KK_dip->SetLineStyle(1);
		//significanceGraph_KK_dip->SetLineWidth(2);
		significanceGraph_KK_dip->SetLineColor(kGreen);
		significanceGraph_KK_dip->Draw("C");
		significanceGraph_E6_dip->SetLineStyle(5);
		//significanceGraph_E6_dip->SetLineWidth(2);
		significanceGraph_E6_dip->SetLineColor(kGreen);
		significanceGraph_E6_dip->Draw("C");

		// 5sigma line
		fiveSigmaLineGraph->SetLineStyle(3);
		fiveSigmaLineGraph->SetLineWidth(2);
		fiveSigmaLineGraph->SetLineColor(kRed);
		fiveSigmaLineGraph->Draw("C");

		leg2 = new TLegend(0.18, 0.25, 0.56, 0.65);
		leg2->AddEntry(significanceGraph_KK_dip,"KK(dip)","L");
		leg2->AddEntry(significanceGraph_KK_peak,"KK(peak)","L");
		leg2->AddEntry(significanceGraph_E6_dip,"Z'_{SM}(dip)","L");
		leg2->AddEntry(significanceGraph_E6_peak,"Z'_{SM}(peak)","L");
		leg2->AddEntry(fiveSigmaLineGraph,"5#sigma","L");
		leg2->Draw();
	}
	else {
		significanceGraph_KK->SetMinimum(1.);
		significanceGraph_KK->SetMaximum(1.e3);
		significanceGraph_KK->SetTitle("");
		significanceGraph_KK->SetLineStyle(1);
		significanceGraph_KK->GetXaxis()->SetTitle("m^{*} [GeV]");
		sigmaTitle = "Significance";
		significanceGraph_KK->GetYaxis()->SetTitle(sigmaTitle.c_str());
		significanceGraph_KK->Draw("AC");
	
		significanceGraph_E6->SetLineStyle(3);
		significanceGraph_E6->Draw("C");

		fiveSigmaLineGraph->SetLineStyle(1);
		fiveSigmaLineGraph->SetLineColor(kRed);
		fiveSigmaLineGraph->Draw("C");
	
		leg2 = new TLegend(0.18, 0.35, 0.56, 0.55);
		leg2->AddEntry(significanceGraph_E6,"Z'_{SM}","L");
		leg2->AddEntry(significanceGraph_KK,"KK","L");
		leg2->AddEntry(fiveSigmaLineGraph,"5#sigma","L");
		leg2->Draw();
	}

	c2->Print(rootFileName2.c_str(),"root");
	c2->Update();
	c2->Print(epsFileName2.c_str());
	rootFile2->Write();

	cout << "done c2" << endl;

	cout << "\n" << "done..." << endl;

	// wait so the plots can hold on
	c1->WaitPrimitive();	
	c2->WaitPrimitive();

	return 0;
}

