// emacs: this is -*- c++ -*-
/*********************************************/
/* INTERFACE CLASSES TO Pythia8              */
/* feed pythia with a completely new process */
/*********************************************/

#ifndef PYTHIAPLUGIN_h
#define PYTHIAPLUGIN_h

//------------------------------
#include "Moses.h"
#include "MosesExamplesCommon.h"
//------------------------------

using namespace std;
using namespace Pythia8; 

namespace Moses1 {

// A derived class for a completely new, user defined (from scratch) process
class new2Process : public Sigma2Process {
	public:
		// Constructor.
//		new2Process(int idIn) : m_idNew(idIn), mevent(NULL) {}
		new2Process(int idIn) : m_idNew(idIn) {}

		// initialize the process:
		void initProc();

		// Calculate flavour-independent parts of cross section.
		virtual void sigmaKin();

		// Evaluate sigmaHat(sHat). 
		virtual double sigmaHat();

		// Select flavour, colour and anticolour.
		virtual void setIdColAcol();

		virtual string	name()		const {return "qqbar->gmZ0->mu+mu-";}
		virtual int	code()		const {return 100002;}
		virtual string	inFlux()	const {return "ffbarSame";}
		virtual int	resonanceA()	const {return 10023;}
		virtual bool    hasIntermediate() const {return true;}
		virtual int     idIntermediate()  const {return 10023;}
		virtual bool	isSChannel()	const {return true;}
		virtual int	id3Mass()	const {return m_idNew;}
		virtual int	id4Mass()	const {return m_idNew;}
		virtual bool	convert2mb()	const {return true;}



//		void registerEvent(Event* e) { mevent = e; }


	private:
		// Store flavour-specific process information and standard prefactor.
		double m_sigma;
		int m_idNew;
		bool m_isPhysical;
		customParticle m_outFermion, m_inFermion;
		dcomplex m_fullAmp;
		double m_totalwidth_Z0;
		double m_mr, m_betaf, m_cosThe;
		double m_thetaDistribution;
		double m_openFracPair;

		parameters m_prm; // create an instance of the parameters ***class***
		physicalParameters m_physPrm; // create an instance of the physical parameters ***struct***
		runParameters m_runPrm; // create an instance of the run parameters ***struct***

		PDTparticle m_pdtPrt; // set the class for PDTparticle call
		CUSTOMparticle m_customPrt;  // set the class for CUSTOMparticle call
		couplings m_cplng; // set the class for couplings call
		partialWidth m_wdt; // set the class for partialWidth call
		helicityAmplitude m_hAmp; // set the class for helicityAmplitude call
		rapidity m_rap;  // set the class for rapidity call
		lhapdf m_lha; // set the class for lhapdf call

		// set a vector to contain all pdtParticle structs of properties
		vector<pdtParticle> m_PDTfermions;
		vector<pdtParticle> m_PDTbosons;
		vector<customParticle> m_CUSTOMfermions; // define a vector of custom particles


                Event* mevent;

};


}
#endif
