// usageExample_HepPDT.cxx is a part of Moses,
// a framework of interfaces for developing HEP models.
// Copyright (C) 2010 Noam Hod and Mark Sutton
//
// Moses is licenced under version 2 of the GPL,
// see COPYING for details.
// Please respect the MCnet academic usage guidelines,
// see http://www.montecarlonet.org/GUIDELINES for details.


// This is an example program to demonstrate the use of PDT
// interface. in particular, it demonstrates the way particle
// properties are being handled.

#include "Moses.h"
#include "MosesExamplesCommon.h"

using namespace Moses1;

int main(int argc, char ** argv) {
	cout << "Holy Moses..." << endl;
	//cout << "Received " << argc << " arguments...\n";
	//for (int i=0; i<argc; i++)
	//{
	//	cout << "argument " << i << ": " << argv[i] << endl;
	//}
	string flag = "";
	string abstract = "";
	if(argc>1) {
		string flag(argv[1]);
		string abstract(argv[2]);
		if (flag=="-s" && abstract=="yes")
		{
			cout << "\t +----------------------- process summary ------------------------+" << endl;
			cout << "\t | (1) description of program:                                    |" << endl;
			cout << "\t |     This is an example program to demonstrate the use of PDT   |" << endl;
			cout << "\t |     interface. in particular, it demonstrates the way particle |" << endl;
			cout << "\t |     properties are being handled.                              |" << endl;
			cout << "\t | (2) description of the user inputs: none                       |" << endl;
			cout << "\t | (3) description of outputs:                                    |" << endl;
			cout << "\t |     - $MOSESDATA/singleParticle.tbl                            |" << endl;
			cout << "\t |     - $MOSESDATA/herwig2pdt.txt                                |" << endl;
			cout << "\t |     - $MOSESDATA/pythia2pdt.txt                                |" << endl;
			cout << "\t |     - $MOSESDATA/pdg2pdt.txt                                   |" << endl;
			cout << "\t +----------------------- process summary ------------------------+" << endl;
			return 0;
		}
	}




	cout << "\n\n\n" << "------------------------Read & Print particle data from Pythia's format in 2 ways------------------------" << "\n\n\n";
	
	// the PDTparticle constructor will set the table of particle properties,
	// it will initialize the "particle" class and will read from the ***supplied*** PDT table file:
	PDTparticle prt;
	prt.setPDTtable("HEPPDTTBL","particle.tbl"); // can also initialize with full path and name
	
	cout << "PDTparticle class was successfuly initialized" << endl;
	cout << "**************************************************" << endl;
	cout << "*** NOTE THAT YOU CAN CALL A SPECIFIC PARTICLE ***" << endl;
	cout << "*** USING PDT ENUM DEFINED IN: mosesParticle.h ***" << endl;
	cout << "**************************************************" << endl;


	// pdt key to fundamental particles:
	// particle: d u s c b t b' t' e- nu_e mu nu_mu tau- nu_tau
	// id:       1 2 3 4 5 6 7  8  11 12   13 14    15   16
	// particle: g  gamma Z0 W+ Higgs0 Z'
	// id:       21 22    23 24 25     32
	
	int id = 111; // pion
	
	cout << "first way: get single property each time" << endl;
	// get a single property at a time of id = 111
	string name = prt.getName(id);
	double mass = prt.getMass(id);
	double charge = prt.getCharge(id);
	double color = prt.getColor(id);
	double Lspin = prt.getSpin(id,"L");
	double Sspin = prt.getSpin(id,"S");
	double Jspin = prt.getSpin(id,"J");
	double width = prt.getWidth(id);
	double lifetime = prt.getLifetime(id);

	// print the properties of id = 111
	cout << "name = " << name << endl;
	cout << "mass = " << mass << endl;
	cout << "charge = " << charge << endl;
	cout << "color = " << color << endl;
	cout << "Lspin = " << Lspin << endl;
	cout << "Sspin = " << Sspin << endl;
	cout << "Jspin = " << Jspin << endl;
	cout << "width = " << width << endl;
	cout << "lifetime = " << lifetime << endl;	

	cout << "\n" << "second way: get all properties in one go" << endl;
	// get/print all at once
	id = 11;


	commonOperations commonOps;
	string tbl = commonOps.checkANDsetFilepath("MOSESDATA","/singleParticle.tbl");
	prt.getAll(id,tbl.c_str()); // writes all properties of id = 111 to "/singleParticle.tbl" file
	
	pdtParticle prop; // particle regular prorerties struct
	bool print = true;
	prt.getAll(id, prop, print); // get all properties in one go and fill the pdtParticle struct
	

	
	cout << "\n\n" << "------------------------get some selected particles and put them into a vector------------------------" << "\n\n";
	print = true;
	// initialize an arbitrary array of id's of the leptons/quarks to be loaded. using "pdtParticle" enum
	const int idFermions[] = {PDTDWN,PDTUP,PDTSTR,PDTCHM,PDTBOT,PDTTOP,PDTE,PDTMU,PDTTAU,PDTNUE,PDTNUMU,PDTNUTAU};
	// d, u, s, c, b, t, e, mu, tau, nu_e, nu_mu, nu_tau
	// set a vector to contain all pdtParticle structs of properties
	vector<pdtParticle> PDTfermions;
	 // fill all quarks and leptons properties in a single vector
	prt.getAll(idFermions, sizeof(idFermions)/sizeof(*idFermions), PDTfermions, print);



	cout << "\n\n" << "------------------------translate some selected particle ids------------------------" << "\n\n";
	int pdtID, herwigID=11, pythiaID=11, pdgID=11;
	
	pdtID = prt.herwig2pdt(herwigID);
	cout << "herwigID=" << herwigID << " translation to pdtID result: " << pdtID << endl;
	herwigID = prt.pdt2herwig(pdtID);
	cout << "and, pdtID=" << pdtID << " translation to herwigID result: " << pdtID << endl;
	prt.herwig2pdt_all("MOSESDATA","/herwig2pdt.txt");
	cout << endl;

	pdtID = prt.pythia2pdt(pythiaID);
	cout << "pythiaID=" << pythiaID << " translation to pdtID result: " << pdtID << endl;
	pythiaID = prt.pdt2pythia(pdtID);
	cout << "and, pdtID=" << pdtID << " translation to pythiaID result: " << pythiaID << endl;
	prt.pythia2pdt_all("MOSESDATA","/pythia2pdt.txt");
	cout << endl;

	pdtID = prt.pdg2pdt(pdgID);
	cout << "pdgID=" << pdgID << " translation to pdtID result: " << pdtID << endl;
	pdgID = prt.pdt2pdg(pdtID);
	cout << "and, pdtID=" << pdtID << " translation to pdgID result: " << pdgID << endl;
	prt.pdg2pdt_all("MOSESDATA","/pdg2pdt.txt");




	cout << "\n\n" << "done..." << endl;
}

