#ifndef MOSESJETFINDERWRAP_h
#define MOSESJETFINDERWRAP_h

#include "Basics.h"
#include "Event.h"
#include "Analysis.h"

#include "fastjet/PseudoJet.hh"
#include "fastjet/ClusterSequence.hh"

#include "mosesCommon.h"

using namespace std;
using namespace Pythia8;

namespace Moses1 {


class mosesClusterJetWrap
{
        public: 
		mosesClusterJetWrap(string measure = "Lund", int select = 2,
				int massSet = 2, bool precluster = false, bool reassign = false) :
				m_measure(measure), m_select(select), m_massSet(massSet),
				m_precluster(precluster), m_reassign(reassign)
		{
			setClusterJetPtr();
		}
                ~mosesClusterJetWrap();
		void setClusterJetPtr();
		void setClusterJetAnalyzer(double yScale, double pTscale, int nJetMin = 1, int nJetMax = 0);
		bool analyzeClusterJet( const Event& event );
		double getClusterJetSettings(string settingsName);
		double getClusterJetAnalyzerSettings(string settingsName);
		ClusterJet* getClusterJet();
				
		void getClusterJetSettings(string settingsName, int& settingsValue);
		void getClusterJetSettings(string settingsName, bool& settingsValue);
		void getClusterJetSettings(string settingsName, string& settingsValue);
		void getClusterJetAnalyzerSettings(string settingsName, double& settingsValue);
		void getClusterJetAnalyzerSettings(string settingsName, int& settingsValue);
        private:
		ClusterJet* m_clusterJet;
				
		string m_measure;
		int m_select;
		int m_massSet;
		bool m_precluster;
		bool m_reassign;
				
		double m_yScale;
		double m_pTscale;
		int m_nJetMin;
		int m_nJetMax;
		bool m_isAnalyzerSet;
};


class mosesCellJetWrap
{
        public:
		mosesCellJetWrap(double etaMax = 5., int nEta = 50, int nPhi = 32,
				int select = 2, int smear = 0, double resolution = 0.5,
				double upperCut = 2., double threshold = 0., Rndm* rndmPtr = 0) :
				m_etaMax(etaMax), m_nEta(nEta), m_nPhi(nPhi), m_select(select),
				m_smear(smear), m_resolution(resolution), m_upperCut(upperCut),
				m_threshold(threshold), m_rndmPtr(rndmPtr)
		{
			setCellJetPtr();
		}
                ~mosesCellJetWrap();
		void setCellJetPtr();
		void setCellJetAnalyzer(double eTjetMin = 20., double coneRadius = 0.7, double eTseed = 1.5);
		bool analyzeCellJet( const Event& event );
		CellJet* getCellJet();
				
		void getCellJetSettings(string settingsName, int& settingsValue);
		void getCellJetSettings(string settingsName, double& settingsValue);
		void getCellJetSettings(string settingsName, Rndm* settingsValue);
		void getCellJetAnalyzerSettings(string settingsName, double& settingsValue);
				
        private:
		CellJet* m_cellJet;
				
		double m_etaMax;
		int m_nEta;
		int m_nPhi;
		int m_select;
		int m_smear;
		double m_resolution;
		double m_upperCut;
		double m_threshold;
		Rndm* m_rndmPtr;
			
		double m_eTjetMin;
		double m_coneRadius;
		double m_eTseed;
		bool m_isAnalyzerSet;
};


class mosesFastJetWrap
{
	public:
		mosesFastJetWrap() { setFastJetPtr(); }
		~mosesFastJetWrap();
		void setFastJetPtr();
		void setFastJetAnalyzer( double Rparam,
								fastjet::Strategy strategy,
								fastjet::RecombinationScheme recombScheme,
								fastjet::JetAlgorithm algorithm
								);
		bool analyzeFastJet( const Event& event );
		fastjet::JetDefinition* getFastJet();

		void getFastJetAnalyzerSettings(string settingsName, double& settingsValue);
		void getFastJetAnalyzerSettings(string settingsName, fastjet::Strategy& settingsValue);
		void getFastJetAnalyzerSettings(string settingsName, fastjet::RecombinationScheme& settingsValue);
		void getFastJetAnalyzerSettings(string settingsName, fastjet::JetAlgorithm& settingsValue);

		vector<fastjet::PseudoJet> inclusiveJets, sortedJets;
				
	private:
		fastjet::JetDefinition *m_jetDef;

		double m_Rparam;
		fastjet::Strategy m_strategy;
		fastjet::RecombinationScheme m_recombScheme;
		fastjet::JetAlgorithm m_jetAlgorithm;
				
		vector<fastjet::PseudoJet> m_fjInputs;
				
		bool m_isAnalyzerSet;
};


}

#endif

