#include "mosesRunSummary.h"

using namespace std;

namespace Moses1 {
	
mosesRunSummary::mosesRunSummary(TFile* file)
{
	m_file = file;
	m_file->cd("runControl");
	
	m_processMap = new multimap<int, string>;
	
	m_measure = new vector<string>;
	
	m_sROOTvs = new vector<string>;
	m_sLHAPDFvs = new vector<string>;
	m_sFASTJETvs = new vector<string>;
	m_sHepPDTvs = new vector<string>;
	m_sPYTHIA8vs = new vector<string>;
	m_sMOSESvs = new vector<string>;
	m_sLHAPDFset = new vector<string>;
	m_sUserName = new vector<string>;
	m_sDate = new vector<string>;
}

mosesRunSummary::~mosesRunSummary() {}


void mosesRunSummary::writeStringFromBash(string cmd)
{
	string s = cmd+" > /tmp/cmdline";
	system( s.c_str() );
}
string mosesRunSummary::readStringFromBash(string cmd)
{
	writeStringFromBash(cmd);

	string fileName = "/tmp/cmdline";
	string str = "";
	
	ifstream fTemp;
	fTemp.open( fileName.c_str() );
	//fTemp >> str;
	getline(fTemp,str);
	string command = "rm -f " + fileName;
	system( command.c_str() );
	
	return str;
}


// called during the event loop
void mosesRunSummary::countProcesses(Pythia& pythia)
{
	m_processMap->insert( pair<int, string>( pythia.info.code(), pythia.info.name() ) );
}

// called at the end of the loop
void mosesRunSummary::fillProcesses()
{	
	m_sProcessName->clear();
	m_nProcessCode->clear();
    m_nProcessCount->clear();
	
	map<int, string>* tmpMap = new map<int, string>;
	
	
	for(multimap<int, string>::iterator it=m_processMap->begin() ; it!=m_processMap->end() ; ++it)
	{
		if( tmpMap->find((*it).first)==tmpMap->end() )
		{
			tmpMap->insert( pair<int, string>( (*it).first, (*it).second ) );
			
			m_nProcessCode->push_back( (*it).first );
			m_sProcessName->push_back( (*it).second );
			m_nProcessCount->push_back( m_processMap->count((*it).first) );
		}
	}
	
	delete tmpMap;
}
void mosesRunSummary::printProcesses()
{
	cout << "all processes:" << endl;
	for(int i=0 ; i<(int)m_nProcessCode->size() ; i++)
	{	
		cout << "\t" << "code = " << (*m_nProcessCode)[i];
		cout << "\t" << "name = " << (*m_sProcessName)[i];
		cout << "\t" << "mult = " << (*m_nProcessCount)[i];
		cout << endl;
	}
}


void mosesRunSummary::buildSwSummaryTree()
{
	m_file->cd("runControl");
    string swHeaderTreeName  = "swHeaderTree";
    string swHeaderTreeTitle = swHeaderTreeName;
	m_swSummaryTree = new TTree(swHeaderTreeName.c_str(), swHeaderTreeTitle.c_str());
	m_swSummaryTree->Branch("ROOTvs",    &m_sROOTvs);
	m_swSummaryTree->Branch("LHAPDFvs",  &m_sLHAPDFvs);	
	m_swSummaryTree->Branch("LHAPDFset", &m_sLHAPDFset);
	m_swSummaryTree->Branch("FASTJETvs", &m_sFASTJETvs);
	m_swSummaryTree->Branch("HepPDTvs",  &m_sHepPDTvs);
	m_swSummaryTree->Branch("PYTHIA8vs", &m_sPYTHIA8vs);
	m_swSummaryTree->Branch("MOSESvs",   &m_sMOSESvs);
	m_swSummaryTree->Branch("UserName",  &m_sUserName);
	m_swSummaryTree->Branch("Date",      &m_sDate);
}



void mosesRunSummary::buildSummaryTree()
{
	m_file->cd("runControl");
    string runHeaderTreeName  = "runHeaderTree";
    string runHeaderTreeTitle = runHeaderTreeName;
	m_runSummaryTree = new TTree(runHeaderTreeName.c_str(), runHeaderTreeTitle.c_str());
	m_beams_id = new vector<int>;
	m_beams_m0 = new vector<double>;
	m_beams_p0 = new vector<double>;
	m_beams_pz = new vector<double>;
	m_sProcessName  = new vector<string>;
	m_nProcessCode  = new vector<int>;
	m_nProcessCount = new vector<int>;
	m_runSummaryTree->Branch("beamParticle_id", &m_beams_id);
	m_runSummaryTree->Branch("beamParticle_m0", &m_beams_m0);	
	m_runSummaryTree->Branch("beamParticle_p0", &m_beams_p0);
	m_runSummaryTree->Branch("beamParticle_pz", &m_beams_pz);
	m_runSummaryTree->Branch("eCM",          &m_eCM);
	m_runSummaryTree->Branch("nEvents",      &m_nEvents);
	m_runSummaryTree->Branch("sProcessName", &m_sProcessName);
	m_runSummaryTree->Branch("nProcessCode", &m_nProcessCode);
	m_runSummaryTree->Branch("nProcessMult",&m_nProcessCount);
	m_runSummaryTree->Branch("nTried",       &m_nTried);
	m_runSummaryTree->Branch("nSelected",    &m_nSelected);
	m_runSummaryTree->Branch("nAccepted",    &m_nAccepted);
	m_runSummaryTree->Branch("sigma_mb",     &m_sigma_mb);
	m_runSummaryTree->Branch("sigmaErr_mb",  &m_sigmaErr_mb);
	m_runSummaryTree->Branch("mHatMin",      &m_nMHatMin);
	m_runSummaryTree->Branch("mHatMax",      &m_nMHatMax);
}

void mosesRunSummary::buildClusterJetSummaryTree()
{
	m_file->cd("runControl");
	string ClusterJetHeaderTreeName  = "ClusterJetHeaderTree";
	string ClusterJetHeaderTreeTitle = ClusterJetHeaderTreeName;
	m_clusterJetSummaryTree = new TTree(ClusterJetHeaderTreeName.c_str(), ClusterJetHeaderTreeTitle.c_str());
	m_clusterJetSummaryTree->Branch("measure",    &m_measure);
	m_clusterJetSummaryTree->Branch("ClusterJetSelect",     &m_select_cluster);
	m_clusterJetSummaryTree->Branch("massSet",    &m_massSet);
	m_clusterJetSummaryTree->Branch("precluster", &m_precluster);
	m_clusterJetSummaryTree->Branch("reassign",   &m_reassign);
	m_clusterJetSummaryTree->Branch("yScale",     &m_yScale);
	m_clusterJetSummaryTree->Branch("pTscale",    &m_pTscale);
	m_clusterJetSummaryTree->Branch("nJetMin",    &m_nJetMin);
	m_clusterJetSummaryTree->Branch("nJetMax",    &m_nJetMax);
}
	
void mosesRunSummary::buildCellJetSummaryTree()
{
	m_file->cd("runControl");
	string CellJetHeaderTreeName  = "CellJetHeaderTree";
	string CellJetHeaderTreeTitle = CellJetHeaderTreeName;
	m_cellJetSummaryTree = new TTree(CellJetHeaderTreeName.c_str(), CellJetHeaderTreeTitle.c_str());
	m_cellJetSummaryTree->Branch("etaMax",     &m_etaMax);
	m_cellJetSummaryTree->Branch("nEta",       &m_nEta);
	m_cellJetSummaryTree->Branch("nPhi",       &m_nPhi);
	m_cellJetSummaryTree->Branch("CellJetSelect",     &m_select_cell);
	m_cellJetSummaryTree->Branch("smear",      &m_smear);
	m_cellJetSummaryTree->Branch("resolution", &m_resolution);
	m_cellJetSummaryTree->Branch("upperCut",   &m_upperCut);
	m_cellJetSummaryTree->Branch("threshold",  &m_threshold);
	m_cellJetSummaryTree->Branch("eTjetMin",   &m_eTjetMin);
	m_cellJetSummaryTree->Branch("coneRadius", &m_coneRadius);
	m_cellJetSummaryTree->Branch("eTseed",     &m_eTseed);
}

void mosesRunSummary::buildFastJetSummaryTree()
{	
	m_file->cd("runControl");
	string FastJetHeaderTreeName  = "FastJetHeaderTree";
	string FastJetHeaderTreeTitle = FastJetHeaderTreeName;
	m_fastJetSummaryTree = new TTree(FastJetHeaderTreeName.c_str(), FastJetHeaderTreeTitle.c_str());
	m_fastJetSummaryTree->Branch("Rparam",       &m_Rparam);
	m_fastJetSummaryTree->Branch("strategy",     &m_strategy);
	m_fastJetSummaryTree->Branch("recombScheme", &m_recombScheme);
	m_fastJetSummaryTree->Branch("jetAlgorithm", &m_jetAlgorithm);
}


void mosesRunSummary::fillSwSummaryTree(string sLHAPDFset)
{
	m_sROOTvs->push_back( readStringFromBash("root-config --version") );
	m_sLHAPDFvs->push_back( readStringFromBash("lhapdf-config --version") );
	m_sLHAPDFset->push_back( sLHAPDFset );
	m_sFASTJETvs->push_back( readStringFromBash("fastjet-config --version") );
	m_sHepPDTvs->push_back( readStringFromBash("moses-config --heppdt_version") );
	m_sPYTHIA8vs->push_back( readStringFromBash("moses-config --pythia8_version") );
	m_sMOSESvs->push_back( readStringFromBash("moses-config --version") );
	m_sUserName->push_back( readStringFromBash("whoami") );
	m_sDate->push_back( readStringFromBash("date") );
	
	m_swSummaryTree->Fill();
}


void mosesRunSummary::fillSummaryTree(Pythia& pythia, int evts)
{
	// once per run
	m_beams_id->clear();
	m_beams_id->push_back( pythia.info.idA() );
	m_beams_id->push_back( pythia.info.idB() );

	m_beams_m0->clear();
	m_beams_m0->push_back( pythia.info.mA() );
    m_beams_m0->push_back( pythia.info.mB() );

	m_beams_p0->clear();
	m_beams_p0->push_back( pythia.info.eA() );
    m_beams_p0->push_back( pythia.info.eB() );

	m_beams_pz->clear();
	m_beams_pz->push_back( pythia.info.pzA() );
    m_beams_pz->push_back( pythia.info.pzB() );
	
	fillProcesses();
	printProcesses();
	
	m_eCM          = pythia.info.eCM();
	m_nEvents      = evts;
	m_nTried       = pythia.info.nTried();
	m_nSelected    = pythia.info.nSelected();
	m_nAccepted    = pythia.info.nAccepted();
	m_sigma_mb     = pythia.info.sigmaGen();
	m_sigmaErr_mb  = pythia.info.sigmaErr();

	m_nMHatMin = pythia.settings.parm("PhaseSpace:mHatMin");
	m_nMHatMax = pythia.settings.parm("PhaseSpace:mHatMax");
	
	m_runSummaryTree->Fill(); // Allways FIll !
}

void mosesRunSummary::fillClusterJetSummaryTree(mosesClusterJetWrap* clusterJetWrap)
{
	if(clusterJetWrap!=NULL)
	{
		string measure;
		clusterJetWrap->getClusterJetSettings("measure",    measure);
		m_measure->push_back(measure);
		clusterJetWrap->getClusterJetSettings("select",     m_select_cluster);
		clusterJetWrap->getClusterJetSettings("massSet",    m_massSet);
		clusterJetWrap->getClusterJetSettings("precluster", m_precluster);
		clusterJetWrap->getClusterJetSettings("reassign",   m_reassign);
		
		clusterJetWrap->getClusterJetAnalyzerSettings("yScale",  m_yScale);
		clusterJetWrap->getClusterJetAnalyzerSettings("pTscale", m_pTscale);
		clusterJetWrap->getClusterJetAnalyzerSettings("nJetMin", m_nJetMin);
		clusterJetWrap->getClusterJetAnalyzerSettings("nJetMax", m_nJetMax);
		
		m_clusterJetSummaryTree->Fill(); // FIll if not NULL!
	}
}

void mosesRunSummary::fillCellJetSummaryTree(mosesCellJetWrap* cellJetWrap)
{
	if(cellJetWrap!=NULL)
	{
		cellJetWrap->getCellJetSettings("etaMax",     m_etaMax);
		cellJetWrap->getCellJetSettings("nEta",       m_nEta);
		cellJetWrap->getCellJetSettings("nPhi",       m_nPhi);
		cellJetWrap->getCellJetSettings("select",     m_select_cell);
		cellJetWrap->getCellJetSettings("smear",      m_smear);
		cellJetWrap->getCellJetSettings("resolution", m_resolution);
		cellJetWrap->getCellJetSettings("upperCut",   m_upperCut);
		cellJetWrap->getCellJetSettings("threshold",  m_threshold);
		
		cellJetWrap->getCellJetAnalyzerSettings("eTjetMin",   m_eTjetMin);
		cellJetWrap->getCellJetAnalyzerSettings("coneRadius", m_coneRadius);
		cellJetWrap->getCellJetAnalyzerSettings("eTseed",     m_eTseed);
		
		m_cellJetSummaryTree->Fill(); // FIll if not NULL!
	}
}

void mosesRunSummary::fillFastJetSummaryTree(mosesFastJetWrap* fastJetWrap)
{
	if(fastJetWrap!=NULL)
	{	
		fastJetWrap->getFastJetAnalyzerSettings("Rparam",   m_Rparam);
		fastJetWrap->getFastJetAnalyzerSettings("strategy", m_strat);
		m_strategy = (int)m_strat;
		fastJetWrap->getFastJetAnalyzerSettings("recombScheme",  m_recSchm);
		m_recombScheme = (int)m_recSchm;
		fastJetWrap->getFastJetAnalyzerSettings("jetAlgorithm", m_jetAlgo);
		m_jetAlgorithm = (int)m_jetAlgo;
		
		m_fastJetSummaryTree->Fill(); // FIll if not NULL!
	}
}


void mosesRunSummary::makeAllTreesFriends()
{
	m_runSummaryTree->AddFriend( "runControl/ClusterJetHeaderTree", m_file );
	m_runSummaryTree->AddFriend( "runControl/CellJetHeaderTree",    m_file );
	m_runSummaryTree->AddFriend( "runControl/FastJetHeaderTree",    m_file );
	
	m_runSummaryTree->AddFriend( "runControl/swHeaderTree",         m_file );
}



void mosesRunSummary::writeSummaryTrees()
{
    m_file->cd("runControl");
    m_runSummaryTree->Write();
    m_swSummaryTree->Write();
	m_clusterJetSummaryTree->Write();
	m_cellJetSummaryTree->Write();
	m_fastJetSummaryTree->Write();
}

}

