/********************************************************/
/* AN INTERFACE CLASS TO HepPDT                         */
/* class to get particle properties from pdt table file */
/* all methods are using HepPDT methods                 */
/********************************************************/
#include <iostream>
#include <stdlib.h>
#include <stdio.h>	// for the sprintf call
#include <string>
#include <sstream>	// for the int to string operation (stringstream call)
#include <cstring>	// for the string functions
#include <math.h>
#include <fstream>
#include <vector>
#include <fstream>
//-------------------------
#include "HepPDT/ParticleID.hh"
#include "HepPDT/ParticleData.hh"
#include "HepPDT/TableBuilder.hh"
#include "HepPDT/ParticleDataTable.hh"
#include "HepPDT/TempParticleData.hh"
#include "HepPDT/SpinState.hh"
//-------------------------
#include "HepPID/ParticleIDTranslations.hh"
#include "HepPID/ParticleIDMethods.hh"
#include "HepPID/ParticleName.hh"
#include "HepPID/Version.hh"
//-------------------------
#include "useHEPPDT.h"
#include "mosesCommon.h"


using namespace std;

namespace Moses1 {

PDTparticle::PDTparticle() {}
PDTparticle::~PDTparticle(){}




void PDTparticle::setPDTtable(string envPath, string fileName) { // initialize with environment variable 
	
	commonOperations commonOps;
	string pdttblFile = commonOps.checkANDsetFilepath(envPath,fileName); // this is a string which ifstream open() will not open
	// have to convert to const char with "pdttblFile.c_str()" and put it as argument for ifstream open()
	
	cout << "reading from tbl file: " << pdttblFile << endl;
	ifstream pdtfile(pdttblFile.c_str()); // and then con
	if(!pdtfile) {
		cerr << "cannot open " << pdttblFile << endl;
		exit(-1);
	}
	{
		HepPDT::TableBuilder tb(m_datacol); // Construct table builder
		if(!addParticleTable(pdtfile,tb)) { // read the input - put as many here as you want
			cerr << "in PDTparticle.cxx: error reading pdt file " << endl;
		}
	} // the tb destructor fills m_datacol !!!
}


void PDTparticle::setPDTtable(const char pdttblFile[]) { // initialize with full path and file name (overload)
	cout << "reading from tbl file: " << pdttblFile << endl;

	ifstream pdtfile(pdttblFile);
	if(!pdtfile) {
		cerr << "cannot open " << pdttblFile << endl;
		exit(-1);
	}
	{
		HepPDT::TableBuilder tb(m_datacol); // Construct table builder
		if(!addParticleTable(pdtfile,tb)) { // read the input - put as many here as you want
			cerr << "in PDTparticle.cxx: error reading pdt file " << endl;
		}
	} // the tb destructor fills m_datacol !!!
}








// In principle, all information in the PDG may be obtained from ParticleData access methods.
void PDTparticle::getAll(int id, const char outfile[]) {
	ofstream wpdtfile(outfile);
	HepPDT::ParticleDataTable db("my Table");
	HepPDT::ParticleData *pd = m_datacol.particle(HepPDT::ParticleID(id));
	pd->write(wpdtfile);
	cout << "properties of particle with id = " << id << " were written into: " << outfile << endl;
}



void PDTparticle::getAll(int id, pdtParticle &prop, bool print) { // overload
	HepPDT::SpinState allspins = m_datacol.particle(HepPDT::ParticleID(id))->spin(); // set all spins
	// fill the properties structure
	prop.id = id;
	prop.name = m_datacol.particle(HepPDT::ParticleID(id))->name();
	prop.mass = m_datacol.particle(HepPDT::ParticleID(id))->mass();
	prop.charge = m_datacol.particle(HepPDT::ParticleID(id))->charge();
	prop.color = m_datacol.particle(HepPDT::ParticleID(id))->color();
	prop.Lspin = allspins.orbAngMom();
	prop.Sspin = allspins.spin();
	prop.Jspin = allspins.totalSpin();
	prop.width = m_datacol.particle(HepPDT::ParticleID(id))->totalWidth();
	prop.lifetime = m_datacol.particle(HepPDT::ParticleID(id))->lifetime();
	
	if (print) {
		cout << "--- all properties of " << prop.name << " ---" << endl;
		cout << "mass = " << prop.mass << endl;
		cout << "charge = " << prop.charge << endl;
		cout << "color = " << prop.color << endl;
		cout << "Lspin = " << prop.Lspin << endl;
		cout << "Sspin = " << prop.Sspin << endl;
		cout << "Jspin = " << prop.Jspin << endl;
		cout << "width = " << prop.width << endl;
		cout << "lifetime = " << prop.lifetime << endl;
		cout << "-----------------------------" << endl;
	}
}

void PDTparticle::getAll(vector<int> id, vector<pdtParticle> &prop, bool print) { // overload 1
	HepPDT::SpinState allspins;
	pdtParticle tmpProp; // temporary structure to be pushed back
	for (int i=0 ; i<(int)id.size() ; i++) {
		// fill the properties structure
		allspins = m_datacol.particle(HepPDT::ParticleID(id[i]))->spin(); // set all spins
		tmpProp.id = id[i];
		tmpProp.name = m_datacol.particle(HepPDT::ParticleID(id[i]))->name();
		tmpProp.mass = m_datacol.particle(HepPDT::ParticleID(id[i]))->mass();
		tmpProp.charge = m_datacol.particle(HepPDT::ParticleID(id[i]))->charge();
		tmpProp.color = m_datacol.particle(HepPDT::ParticleID(id[i]))->color();
		tmpProp.Lspin = allspins.orbAngMom();
		tmpProp.Sspin = allspins.spin();
		tmpProp.Jspin = allspins.totalSpin();
		tmpProp.width = m_datacol.particle(HepPDT::ParticleID(id[i]))->totalWidth();
		tmpProp.lifetime = m_datacol.particle(HepPDT::ParticleID(id[i]))->lifetime();
		
		prop.push_back(tmpProp);
		
		if (print) {
			cout << "--- all properties of " << prop[i].name << " ---" << endl;
			cout << "mass = " << prop[i].mass << endl;
			cout << "charge = " << prop[i].charge << endl;
			cout << "color = " << prop[i].color << endl;
			cout << "Lspin = " << prop[i].Lspin << endl;
			cout << "Sspin = " << prop[i].Sspin << endl;
			cout << "Jspin = " << prop[i].Jspin << endl;
			cout << "width = " << prop[i].width << endl;
			cout << "lifetime = " << prop[i].lifetime << endl;
			cout << "-----------------------------" << endl;
		}
	}
}



void PDTparticle::getAll(const int *id, int id_length, vector<pdtParticle> &prop, bool print) { // overload 2
	HepPDT::SpinState allspins;
	pdtParticle tmpProp; // temporary structure to be pushed back
	for (int i=0 ; i<id_length ; i++) {
		// fill the properties structure
		allspins = m_datacol.particle(HepPDT::ParticleID(id[i]))->spin(); // set all spins
		tmpProp.id = id[i];
		tmpProp.name = m_datacol.particle(HepPDT::ParticleID(id[i]))->name();
		tmpProp.mass = m_datacol.particle(HepPDT::ParticleID(id[i]))->mass();
		tmpProp.charge = m_datacol.particle(HepPDT::ParticleID(id[i]))->charge();
		tmpProp.color = m_datacol.particle(HepPDT::ParticleID(id[i]))->color();
		tmpProp.Lspin = allspins.orbAngMom();
		tmpProp.Sspin = allspins.spin();
		tmpProp.Jspin = allspins.totalSpin();
		tmpProp.width = m_datacol.particle(HepPDT::ParticleID(id[i]))->totalWidth();
		tmpProp.lifetime = m_datacol.particle(HepPDT::ParticleID(id[i]))->lifetime();
		
		prop.push_back(tmpProp);
		
		if (print) {
			cout << "--- all properties of " << prop[i].name << " ---" << endl;
			cout << "mass = " << prop[i].mass << endl;
			cout << "charge = " << prop[i].charge << endl;
			cout << "color = " << prop[i].color << endl;
			cout << "Lspin = " << prop[i].Lspin << endl;
			cout << "Sspin = " << prop[i].Sspin << endl;
			cout << "Jspin = " << prop[i].Jspin << endl;
			cout << "width = " << prop[i].width << endl;
			cout << "lifetime = " << prop[i].lifetime << endl;
			cout << "-----------------------------" << endl;
		}
	}
}






string PDTparticle::getName(int id) {
	string sname = m_datacol.particle(HepPDT::ParticleID(id))->name();
	return sname;
}
double PDTparticle::getMass(int id) { // can be of type Mesurment
	double mass = m_datacol.particle(HepPDT::ParticleID(id))->mass();
	return(mass);
}
double PDTparticle::getCharge(int id) {
	double chrg = m_datacol.particle(HepPDT::ParticleID(id))->charge();
	return(chrg);
}
double PDTparticle::getColor(int id) {
	double clr = m_datacol.particle(HepPDT::ParticleID(id))->color();
	return(clr);
}
double PDTparticle::getSpin(int id, string state) { // can be of type SpinState
	HepPDT::SpinState allspins = m_datacol.particle(HepPDT::ParticleID(id))->spin();
	double spn = 0;
	if (state == "L") {spn = allspins.orbAngMom();}
	else if (state == "S") {spn = allspins.spin();}
	else if (state == "J") {spn = allspins.totalSpin();}
	else {cerr << "getSpin '" << state << "' is not a valid option" << endl; exit(-1);}
	return(spn);
}
double PDTparticle::getWidth(int id) {
	double wdt = m_datacol.particle(HepPDT::ParticleID(id))->totalWidth();
	return(wdt);
}
double PDTparticle::getLifetime(int id) {
	double lftm = m_datacol.particle(HepPDT::ParticleID(id))->lifetime();
	return(lftm);
}




// ID translations methods:
int PDTparticle::herwig2pdt(const int herwigID) {return(translateHerwigtoPDT(herwigID));}

void PDTparticle::herwig2pdt_all(string envPath, string fileName) {
	commonOperations commonOps;
	string fullPath = commonOps.checkANDsetFilepath(envPath,fileName); // this is a string which ifstream open() will not open
	// have to convert to const char with "fullPath.c_str()" and put it as argument for ifstream open()
	filebuf translationFile;
	translationFile.open(fullPath.c_str(),ios::out);
	ostream os(&translationFile);
	writeHerwigTranslation(os);
	translationFile.close();
	cout << "translation was written to: " << fullPath << endl;
}

int PDTparticle::pdt2herwig(const int pdtID){return (translatePDTtoHerwig(pdtID));}




int PDTparticle::pythia2pdt(const int pythiaID){return(translatePythiatoPDT(pythiaID));}

void PDTparticle::pythia2pdt_all(string envPath, string fileName) {
	commonOperations commonOps;
	string fullPath = commonOps.checkANDsetFilepath(envPath,fileName); // this is a string which ifstream open() will not open
	// have to convert to const char with "fullPath.c_str()" and put it as argument for ifstream open()
	filebuf translationFile;
	translationFile.open(fullPath.c_str(),ios::out);
	ostream os(&translationFile);
	writePythiaTranslation(os);
	translationFile.close();
	cout << "translation was written to: " << fullPath << endl;
}

int PDTparticle::pdt2pythia(const int pdtID){return (translatePDTtoPythia(pdtID));}




int PDTparticle::pdg2pdt(const int pdgID){return(translatePDGtabletoPDT(pdgID));}

void PDTparticle::pdg2pdt_all(string envPath, string fileName) {
	commonOperations commonOps;
	string fullPath = commonOps.checkANDsetFilepath(envPath,fileName); // this is a string which ifstream open() will not open
	// have to convert to const char with "fullPath.c_str()" and put it as argument for ifstream open()
	filebuf translationFile;
	translationFile.open(fullPath.c_str(),ios::out);
	ostream os(&translationFile);
	writePDGTranslation(os);
	translationFile.close();
	cout << "translation was written to: " << fullPath << endl;
}

int PDTparticle::pdt2pdg(const int pdtID){return (translatePDTtoPDGtable(pdtID));}





}
