/************************************/
/* AN INTERFACE CLASS TO LHAPDFWrap */
/* class to evaluate the LHA PDFs   */
/************************************/
#include <iostream>
#include <stdlib.h>
#include <stdio.h>	// for the sprintf call
#include <string>
#include <sstream>	// for the int to string operation (stringstream call)
#include <cstring>	// for the string functions
#include <math.h>
#include <fstream>
#include <vector>
//---------------------
#include "useLHAPDF.h"
#include "mosesCommon.h"
using namespace std;

namespace Moses1 {

lhapdf::lhapdf(){
	//index: -6    -5    -4    -3    -2    -1    0   1 2 3 4 5 6
	//parton: tbar  bbar  cbar  sbar  ubar  dbar glu d u s c b t
	m_allflavorkey.push_back("tbar");
	m_allflavorkey.push_back("bbar");
	m_allflavorkey.push_back("cbar");
	m_allflavorkey.push_back("sbar");
	m_allflavorkey.push_back("ubar");
	m_allflavorkey.push_back("dbar");
	m_allflavorkey.push_back("glu");
	m_allflavorkey.push_back("d");
	m_allflavorkey.push_back("u");
	m_allflavorkey.push_back("s");
	m_allflavorkey.push_back("c");
	m_allflavorkey.push_back("b");
	m_allflavorkey.push_back("t");
}
lhapdf::~lhapdf(){}

// to return a single flavor
void lhapdf::keylhapdf(int flavor, string &key){
	key = m_allflavorkey[flavor+6];
}

// to return all flavors
void lhapdf::keylhapdf(vector<string> &allkeys, bool print){
	allkeys = m_allflavorkey;
	if(print){
		cout << "----" << endl;
		for (int i=0 ; i<(int)allkeys.size() ; i++) cout << allkeys[i] << endl;
		cout << "----" << endl;
	}
}


void lhapdf::setlhapdf(string envPath, string fileName, int subset){ // initialize with environment variable

	commonOperations commonOps;
	string strPath = commonOps.checkANDsetFilepath(envPath,fileName); // checkANDsetFilepath(string,string) returns string,
	const char *temp = strPath.c_str(); // .c_str converts the string to (const char*),
	char *pdfsetFile = NULL; // this is what we want to pass to initPDFSet(char*),
	pdfsetFile = (char*)temp; // so, have to cast the 'temp' from (const char*) to (char*) and put it into 'pdfsetFile'

	cout << "reading from pdf set file: " << pdfsetFile << endl;

	LHAPDF::initPDFSet(pdfsetFile);
	LHAPDF::initPDF(subset);
	
	m_subset = subset;
	
	m_hasXminWarningAppearead = false;
	m_hasXmaxWarningAppearead = false;
	m_hasQ2minWarningAppearead = false;
	m_hasQ2maxWarningAppearead = false;
}


void lhapdf::setlhapdf(string filename, int subset){ // initialize with full path and name (overload)
	cout << "Reading from: " << filename << endl;
	char name[200]; // 'name' will be the name of the grid or data file of the desired set.
	name[0] = '\0';
	strcpy(name,filename.c_str());

	LHAPDF::initPDFSet(name);
	LHAPDF::initPDF(subset);
	
	m_subset = subset;
	
	m_hasXminWarningAppearead = false;
	m_hasXmaxWarningAppearead = false;
	m_hasQ2minWarningAppearead = false;
	m_hasQ2maxWarningAppearead = false;
}



// to return a single flavor
void lhapdf::getlhapdf(double x, double q, int flavor, double &xpdfx) {

	checkQ2(q*q);
	checkX(x);

	xpdfx = LHAPDF::xfx(x,q,flavor);
	//if (xpdfx<0.) cout << "xpdfx=" << xpdfx << endl;
	// returns xf(x,q) for flavour fl - this time the flavour encoding
	// is as in the LHAPDF manual...
	// -6..-1 = tbar,...,ubar, dbar
	// 1..6 = duscbt
	// 0 = g
}


// to return all flavors
void lhapdf::getlhapdf(double x, double q, vector<double> &allxpdfx) { 

	checkQ2(q*q);
	checkX(x);

	allxpdfx = LHAPDF::xfx(x,q); // Update ALL PDF flavours.
	// returns a vector xf(x, Q) with index 0 <= i <= 12.
	// 0..5 = tbar, ..., ubar, dbar; 
	// 6 = g; 
	// 7..12 = d, u, ..., t
}



double lhapdf::getXmin()
{
	return LHAPDF::getXmin(m_subset);
}
double lhapdf::getXmax()
{
	return LHAPDF::getXmax(m_subset);
}
double lhapdf::getQ2min()
{
	return LHAPDF::getQ2min(m_subset);
}
double lhapdf::getQ2max()
{
	return LHAPDF::getQ2max(m_subset);
}
void lhapdf::checkX(double x)
{
	if(x<getXmin())
	{
		if(!m_hasXminWarningAppearead)
		{
			cout << "WARNING: value of x(" << x << ") exceeds the minimum value considered valid for this set, as specified by the set authors." << endl;
			cout << "WARNING: surpressing further massages of this type." << endl;
			m_hasXminWarningAppearead = true;
		}
	}
	if(x>getXmax())
	{
		if(!m_hasXmaxWarningAppearead)
		{
			cout << "WARNING: value of x(" << x << ") exceeds the maximum value considered valid for this set, as specified by the set authors." << endl;
			cout << "WARNING: surpressing further massages of this type." << endl;
			m_hasXmaxWarningAppearead = true;
		}
	}
}
void lhapdf::checkQ2(double Q2)
{
	if(Q2<getQ2min())
	{
		if(!m_hasQ2minWarningAppearead)
		{
			cout << "WARNING: value of Q2(" << Q2 << ") exceeds the minimum value considered valid for this set, as specified by the set authors." << endl;
			cout << "WARNING: surpressing further massages of this type." << endl;
			m_hasQ2minWarningAppearead = true;
		}
	}
	if(Q2>getQ2max())
	{
		if(!m_hasQ2maxWarningAppearead)
		{
			cout << "WARNING: value of Q2(" << Q2 << ") exceeds the maximum value considered valid for this set, as specified by the set authors." << endl;
			cout << "WARNING: surpressing further massages of this type." << endl;
			m_hasQ2maxWarningAppearead = true;
		}
	}
}




// to print a single flavor
void lhapdf::prntlhapdf(double &xpdfx) {
	cout << xpdfx << endl;
}

// to print all flavors
void lhapdf::prntlhapdf(vector<double> &allxpdfx) {
	for (int i=0 ; i<(int)allxpdfx.size() ; i++) {cout << allxpdfx[i] << endl;}
}



double lhapdf::simpsonTest(int nsegments, double s) { // to check if integrate(sum(x*pdf(x))*dx) = 1
	// the Simpson integarting rule:
	// integral(f(x)dx) between [a,b] = (h/3)*( f(x0) + 4f(x1)+2f(x2)+4f(x3)+2f(x4)+...+2f(x[n-2])+4f(x[n-1]) + f(xn) )
	// where: x_n = a + n*h
	// and: h is the step length

	double integrand = 0;
	vector<double> allxpdfx;
	double I=0;
	double dx = 1./nsegments;
	double h = dx/3;
	double x = 0; // lower x

	for (int i=0 ; i<nsegments ; i++) {
		getlhapdf(x,sqrt(s),allxpdfx);
		integrand = 0;
		for (int j=0 ; j<(int)allxpdfx.size() ; j++) {integrand += allxpdfx[j];}
		if (i==0 || i==(nsegments-1)) {I += h*integrand;}
		if (i%2==0 && i!=0 && i!=(nsegments-1)) {I += 2*h*integrand;}
		else {I += 4*h*integrand;}
		x += dx;
	}
	return(I);
}





}
